@@ -8,81 +8,32 @@ import logging
 from os.path import join, exists, split
 from os import remove
 import sqlite3
-from types import NoneType
 
+from common import CACHE_SUBDIR
+from exon import Exon
 from gene import Gene
-from gtf import load_gtf_as_dataframe
-from locus import normalize_chromosome
-import memory_cache
+from gtf import GTF
+from locus import normalize_chromosome, normalize_strand
+from reference_transcripts import ReferenceTranscripts
+from release_info import check_release_number
 from transcript import Transcript
+from url_templates import ENSEMBL_FTP_SERVER
 
 import datacache
 import numpy as np
 import pandas as pd
 
 
-MIN_ENSEMBL_RELEASE = 48
-MAX_ENSEMBL_RELEASE = 77
-
-def _check_release(release):
-    """
-    Convert a user-provided release number into
-    an integer, check to make sure it's in the
-    valid range of Ensembl releases
-    """
-    try:
-        release = int(release)
-    except:
-        raise ValueError("Invalid Ensembl release: %s" % release)
-
-    if release < MIN_ENSEMBL_RELEASE or release > MAX_ENSEMBL_RELEASE:
-        raise ValueError(
-            "Invalid Ensembl releases %d, must be between %d and %d" %
-                (release, MIN_ENSEMBL_RELEASE, MAX_ENSEMBL_RELEASE))
-    return release
-
-# mapping from Ensembl release to which reference assembly it uses
-_human_references = {}
-
-# Ensembl release 48-54 use NCBI36 as a reference
-for i in xrange(48,55):
-    _human_references[i] = 'NCBI36'
-
-# Ensembl releases 55-75 use GRCh37 as a reference
-for i in xrange(55,76):
-    _human_references[i] = 'GRCh37'
-
-# Ensembl releases 76 and 77 use GRCh38
-for i in xrange(76,78):
-    _human_references[i] = 'GRCh38'
-
-def which_human_reference(release):
-    release = _check_release(release)
-    if release not in _human_references:
-        raise ValueError(
-            "No reference found for release %d" % release)
-    return _human_references[release]
-
-# directory which contains GTF files, missing the release number
-URL_DIR_TEMPLATE = 'ftp://ftp.ensembl.org/pub/release-%d/gtf/homo_sapiens/'
-FILENAME_TEMPLATE = "Homo_sapiens.%s.%d.gtf.gz"
-CACHE_SUBDIR = "ensembl"
-
 class EnsemblRelease(object):
 
-    def __init__(self, release, lazy_loading=True):
-        self.release = _check_release(release)
-        self.gtf_url_dir = URL_DIR_TEMPLATE % self.release
-        self.reference_name =  which_human_reference(self.release)
-        self.gtf_filename = FILENAME_TEMPLATE  % (
-            self.reference_name, self.release
-        )
-        self.gtf_url = join(self.gtf_url_dir, self.gtf_filename)
-        self._local_gtf_path = None
-
-        # lazily load DataFrame of all GTF entries if necessary
-        # for database construction
-        self._df_cache = {}
+    def __init__(self, release, lazy_loading=True, server=ENSEMBL_FTP_SERVER):
+        self.cache = datacache.Cache(CACHE_SUBDIR)
+        self.release = check_release_number(release)
+        self.species = "homo_sapiens"
+        self.server = server
+        self.gtf = GTF(self.release, self.species, server)
+        self.reference = ReferenceTranscripts(
+            self.release, self.species, server)
 
         self.logger = logging.getLogger()
         self.logger.setLevel(logging.INFO)
@@ -94,157 +45,39 @@ class EnsemblRelease(object):
             self._db = self._connect_or_create_database()
 
     def __str__(self):
-        return "EnsemblRelease(release=%d, gtf_url='%s')" % (
-            self.release, self.gtf_url)
+        return "EnsemblRelease(release=%d, gtf_url='%s', fasta_url='%s')" % (
+            self.release, self.gtf.url, self.reference.url)
 
     def __repr__(self):
         return str(self)
 
-    def base_gtf_filename(self):
-        """
-        Trim extensions such as ".gtf" or ".gtf.gz",
-        leaving only the base filename which should be used
-        to construct other derived filenames for cached data.
-        """
-        assert ".gtf" in self.gtf_filename, \
-            "GTF filename must contain .gtf extension: %s" % self.gtf_filename
-        parts = self.gtf_filename.split(".gtf")
-        return parts[0]
-
     def _delete_cached_files(self):
-        base = self.base_gtf_filename()
+        """
+        Any files which start with the same name as our GTF file
+        is assumed to be some view of the this release's data and thus
+        safe to delete.
+        """
+        base = self.gtf.base_filename()
         dirpath = self.local_gtf_dir()
         for path in glob(join(dirpath, base + "*")):
             logging.info("Deleting cached file %s", path)
             remove(path)
 
-    def clear_cached_data():
-        self._df_cache = {}
+    def clear_cache(self):
+        self.gtf.clear_cache()
+        self.reference.clear_cache()
         self._delete_cached_files()
-        # TODO: combine caching of parsed CSV files and
-        # caching of partial reads from the database
-        memory_cache.clear_cached_objects()
-
-
-    def local_gtf_path(self):
-        """
-        Returns local path to GTF file for given release of Ensembl,
-        download from the Ensembl FTP server if not already cached.
-        """
-        if self._local_gtf_path is None:
-            self._local_gtf_path = datacache.fetch_file(
-                self.gtf_url,
-                filename=self.gtf_filename,
-                decompress=False,
-                subdir=CACHE_SUBDIR)
-        assert self._local_gtf_path is not None
-        return self._local_gtf_path
-
-    def local_gtf_dir(self):
-        return split(self.local_gtf_path())[0]
-
-    def local_csv_path(self, contig=None, feature=None):
-        """
-        Path to CSV which the annotation data with expanded columns
-        for optional attributes.
-
-        Parameters:
-
-        contig : str, optional
-            Path for subset of data restricted to given contig
-
-        feature : str, optional
-            Path for subset of data restrict to given feature
-        """
-        base = self.base_gtf_filename()
-        dirpath = self.local_gtf_dir()
-        csv_filename = base + ".expanded"
-        if contig:
-            contig = normalize_chromosome(contig)
-            csv_filename += ".contig.%s" % (contig,)
-        if feature:
-            csv_filename += ".feature.%s" % (feature,)
-        csv_filename += ".csv"
-        return join(dirpath, csv_filename)
 
     def local_db_filename(self):
-        base = self.base_gtf_filename()
+        base = self.gtf.base_filename()
         return base + ".db"
 
     def local_db_path(self):
-        dirpath = self.local_gtf_dir()
+        dirpath = self.gtf.local_dir()
         filename = self.local_db_filename()
         return join(dirpath, filename)
 
-    def _load_full_dataframe_from_gtf(self):
-        """
-        Parse this release's GTF file and load it as a Pandas DataFrame
-        """
-        path = self.local_gtf_path()
-        return load_gtf_as_dataframe(path)
-
-    def _load_full_dataframe(self):
-        """
-        Loads full dataframe from cached CSV or constructs it from GTF
-        """
-        csv_path = self.local_csv_path()
-        return memory_cache.load_csv(
-            csv_path, self._load_full_dataframe_from_gtf)
-
-    def dataframe(self, contig=None, feature=None):
-        """
-        Load Ensembl entries as a DataFrame, optionally restricted to
-        particular contig or feature type.
-        """
-
-        if contig:
-            contig = normalize_chromosome(contig)
-
-        if not isinstance(contig, (NoneType, str, unicode)):
-            raise TypeError(
-                "Expected contig to be string, got %s : %s" % (
-                    contig, type(contig)))
-
-        if not isinstance(feature, (NoneType, str, unicode)):
-            raise TypeError(
-                    "Expected feature to be string, got %s : %s" % (
-                        feature, type(feature)))
-
-        key = (contig, feature)
-
-        if key not in self._df_cache:
-            csv_path = self.local_csv_path(contig=contig, feature=feature)
-
-            def local_loader_fn():
-                full_df = self._load_full_dataframe()
-                assert len(full_df) > 0, \
-                    "Dataframe representation of Ensembl database empty!"
-
-                # rename since we're going to be filtering the entries but
-                # may still want to access the full dataset
-                df = full_df
-                if contig:
-                    df = df[df.seqname == contig]
-                    if len(df) == 0:
-                        raise ValueError("Contig not found: %s" % (contig,))
-
-                if feature:
-                    df = df[df.feature == feature]
-                    if len(df) == 0:
-                        # check to make sure feature was somewhere in
-                        # the full dataset before returning an empty dataframe
-                        features = full_df.feature.unique()
-                        if feature not in features:
-                            raise ValueError(
-                                "Feature not found: %s" % (feature,))
-                return df
-
-            self._df_cache[key] = memory_cache.load_csv(
-                csv_path, local_loader_fn)
-
-        return self._df_cache[key]
-
-    def _database_indices(available_columns):
+    def _database_indices(self, available_columns):
         """
         If a release is missing some column we want to index on,
         we have to drop any indices which use that column.
@@ -285,7 +118,7 @@ class EnsemblRelease(object):
     def _create_database(self):
         print "Creating database: %s" % self.local_db_path()
         filename = self.local_db_filename()
-        df = self.dataframe()
+        df = self.gtf.dataframe()
 
         available_columns = set(df.columns)
         indices = self._database_indices(available_columns)
@@ -312,6 +145,7 @@ class EnsemblRelease(object):
                 return db
         return self._create_database()
 
+    @property
     def db(self):
         """
         Return the sqlite3 database for this Ensembl release
@@ -332,46 +166,7 @@ class EnsemblRelease(object):
         # find genes whose start/end boundaries overlap with the position
         return values[overlap_start & overlap_end]
 
-    def dataframe_column_at_locus(
-            self,
-            column_name,
-            contig,
-            start,
-            end=None,
-            offset=None):
-        """
-        Subset of entries which overlap an inclusive range of loci
-        """
-        if end is None and offset is None:
-            end = position
-        elif offset is None:
-            end = position + offset - 1
 
-        df_contig = self.dataframe(contig=contig)
-
-        assert column_name in df_contig, \
-            "Unknown Ensembl property: %s" % column_name
-
-        return self._slice_column(
-            df_contig[column_name], df_contig.start, df_contig.end, start, end)
-
-    def dataframe_at_locus(self, contig, position, end=None, offset=None):
-        """
-        Subset of entries which overlap an inclusive range of
-        chromosomal positions
-        """
-        if end is None and offset is None:
-            end = position
-        elif offset is None:
-            end = position + offset - 1
-
-        df_contig = self.dataframe(contig=contig)
-
-        # find genes whose start/end boundaries overlap with the position
-        overlap_start = df_contig.start <= end
-        overlap_end = df_contig.end >= position
-        overlap = overlap_start & overlap_end
-        return df_contig[overlap]
 
     def _db_columns(self, db):
         table_info = db.execute("PRAGMA table_info(ensembl);").fetchall()
@@ -380,12 +175,24 @@ class EnsemblRelease(object):
     def _db_column_exists(self, db, column_name):
         return column_name in self._db_columns(db)
 
-    def column_at_locus(self, column_name, contig, position, end=None):
+    def column_values_at_locus(
+            self,
+            column_name,
+            feature,
+            contig,
+            position,
+            end=None,
+            strand=None,
+            distinct=False,
+            sorted=False):
         """
         Get the non-null values of a column from the database
         at a particular range of loci
         """
 
+        # TODO: combine with the query method, since they overlap
+        # significantly
+
         if not isinstance(column_name, (str, unicode)):
             raise TypeError(
                 "Expected column_name to be str, got %s : %s" % (
@@ -405,78 +212,165 @@ class EnsemblRelease(object):
             raise TypeError(
                 "Expected end to be integer, got %s : %s" % (end, type(end)))
 
-        db = self.db()
-
-        if not self._db_column_exists(db, column_name):
+        if not self._db_column_exists(self.db, column_name):
             raise ValueError("Unknown Ensembl property: %s" % (column_name,))
 
+        if distinct:
+            distinct_string = "DISTINCT "
+        else:
+            distinct_string = ""
+
         query = """
-            select %s
-            from ensembl
-            where
-                seqname='%s'
-                and start <= %d
-                and end >= %d
-        """  % (column_name, contig, end, position)
-        # self.logger.info("Running query: %s" % query)
-        results = db.execute(query).fetchall()
+            SELECT %s%s
+            FROM ensembl
+            WHERE feature = ?
+            AND seqname=?
+            AND start <= ?
+            AND end >= ?
+
+        """  % (distinct_string, column_name,)
+
+        query_params = [feature, contig, end, position]
+
+        if strand:
+            query += " AND strand = ?"
+            query_params.append(strand)
+
+        tuples = self.db.execute(query, query_params).fetchall()
         # each result is a tuple, so pull out its first element
-        return [result[0] for result in results if result[0] is not None]
+        results = [t[0] for t in tuples if t[0] is not None]
+        if sorted:
+            results.sort()
+        return results
 
 
-    # TODO: add optional 'strand' parameter
-    def _property_values_at_locus(
-            self, property_name, contig, position, end=None):
-        col = self.column_at_locus(
-            property_name,
+    def distinct_column_values_at_locus(
+            self,
+            column,
+            feature,
             contig,
             position,
-            end=end)
-        return list(sorted({c for c in col if c}))
+            end=None,
+            strand=None):
+        """
+        Gather all the distinct values for a property/column at some specified
+        locus.
 
-    def genes_at_locus(self, contig, position, end=None):
-        gene_ids = self.gene_ids_at_locus(contig, position, end=end)
+        Parameters
+        ----------
+        column : str
+            Which property are we getting the values of.
+
+        feature : str
+            Which type of entry (e.g. transcript, exon, gene) is the property
+            associated with?
+
+        contig : str
+            Chromosome or unplaced contig name
+
+        position : int
+            Chromosomal position
+
+        end : int, optional
+            End position of a range, if unspecified assume we're only looking
+            at the single given position.
+
+        strand : str, optional
+            Either the positive ('+') or negative strand ('-'). If unspecified
+            then check for values on either strand.
+        """
+        return self.column_values_at_locus(
+            column,
+            feature,
+            contig,
+            position,
+            end=end,
+            strand=strand,
+            distinct=True,
+            sorted=True)
+
+    def genes_at_locus(self, contig, position, end=None, strand=None):
+        gene_ids = self.gene_ids_at_locus(
+            contig, position, end=end, strand=strand)
         return [self.gene_by_id(gene_id) for gene_id in gene_ids]
 
-    def transcripts_at_locus(self, contig, position, end=None):
+    def transcripts_at_locus(self, contig, position, end=None, strand=None):
         transcript_ids = self.transcript_ids_at_locus(
-            contig, position, end=end)
+            contig, position, end=end, strand=strand)
         return [
             self.transcript_by_id(transcript_id)
             for transcript_id in transcript_ids
         ]
 
-    def gene_ids_at_locus(self, contig, position, end=None):
-        return self._property_values_at_locus(
-            'gene_id', contig, position, end=end)
+    def exons_at_locus(self, contig, position, end=None, strand=None):
+        exon_ids = self.exon_ids_at_locus(
+            contig, position, end=end, strand=strand)
+        return [
+            self.exon_by_id(exon_id)
+            for exon_id in exon_ids
+        ]
 
-    def gene_names_at_locus(self, contig, position, end=None):
-        return self._property_values_at_locus(
-             'gene_name', contig, position, end=end)
+    def gene_ids_at_locus(self, contig, position, end=None, strand=None):
+        return self.distinct_column_values_at_locus(
+            column='gene_id',
+            feature='gene',
+            contig=contig,
+            position=position,
+            end=end,
+            strand=strand)
 
-    def exon_ids_at_locus(self, contig, position, end=None):
-        return self._property_values_at_locus(
-            'exon_id', contig, position, end=end)
+    def gene_names_at_locus(self, contig, position, end=None, strand=None):
+        return self.distinct_column_values_at_locus(
+             column='gene_name',
+             feature='gene',
+             contig=contig,
+             position=position,
+             end=end,
+             strand=strand)
 
-    def transcript_ids_at_locus(self, contig, position, end=None):
-        return self._property_values_at_locus(
-            'transcript_id', contig, position, end=end)
+    def exon_ids_at_locus(self, contig, position, end=None, strand=None):
+        return self.distinct_column_values_at_locus(
+            column='exon_id',
+            feature='exon',
+            contig=contig,
+            position=position,
+            end=end,
+            strand=strand)
 
-    def transcript_names_at_locus(self, contig, position, end=None):
-        return self._property_values_at_locus(
-            'transcript_name', contig, position, end=end)
+    def transcript_ids_at_locus(self, contig, position, end=None, strand=None):
+        return self.distinct_column_values_at_locus(
+            column='transcript_id',
+            feature='transcript',
+            contig=contig,
+            position=position,
+            end=end,
+            strand=strand)
 
-    def protein_ids_at_locus(self, contig, position, end=None):
-        return self._property_values_at_locus(
-            'protein_id', contig, position, end=end)
+    def transcript_names_at_locus(
+            self, contig, position, end=None, strand=None):
+        return self.distinct_column_values_at_locus(
+            column='transcript_name',
+            feature='transcript',
+            contig=contig,
+            position=position,
+            end=end,
+            strand=strand)
 
-    def run_sql_query(self, sql, required=False):
+    def protein_ids_at_locus(self, contig, position, end=None, strand=None):
+        return self.distinct_column_values_at_locus(
+            column='protein_id',
+            feature='transcript',
+            contig=contig,
+            position=position,
+            end=end,
+            strand=strand)
+
+    def run_sql_query(self, sql, required=False, query_params=[]):
         """
         Given an arbitrary SQL query, run it against the Ensembl database
         and return the results.
         """
-        db = self.db()
-        cursor = db.execute(sql)
+        cursor = self.db.execute(sql, query_params)
         results = cursor.fetchall()
         if required and len(results) == 0:
             raise ValueError(
@@ -496,29 +390,75 @@ class EnsemblRelease(object):
         filtered both by the feature type and a user-provided column/value.
         """
         query = """
-            select %s%s
-            from ensembl
-            where
-                %s = '%s' and
-                feature='%s'
+            SELECT %s%s
+            FROM ensembl
+            WHERE %s = ?
+            AND feature= ?
         """ % ("distinct " if distinct else "",
                ", ".join(select_column_names),
-               filter_column,
-               filter_value,
-               feature)
-        return self.run_sql_query(query, required=required)
+               filter_column)
+        query_params = [filter_value, feature]
+        return self.run_sql_query(
+            query, required=required, query_params=query_params)
 
-    def _query_feature(self, column, feature, distinct=True):
+    def _query_feature_values(
+            self,
+            column,
+            feature,
+            distinct=True,
+            contig=None):
         """
         Run a SQL query against the ensembl sqlite3 database, filtered
         only on the feature type.
         """
-        query = "select %s%s from ensembl where feature='%s'" % (
-               "distinct " if distinct else "",
-               column,
-               feature
-        )
-        return self.run_sql_query(query)
+        query = """
+            SELECT %s%s
+            FROM ensembl
+            WHERE feature=?
+        """ % ("DISTINCT " if distinct else "", column)
+        query_params = [feature]
+
+        if contig:
+            contig = normalize_chromosome(contig)
+            query += " AND seqname = ?"
+            query_params.append(contig)
+
+        rows = self.run_sql_query(query, query_params = query_params)
+        return [row[0] for row in rows if row is not None]
+
+    def _all_feature_values(self, column, feature, distinct=True, contig=None):
+        """
+        Cached lookup of all values for a particular feature property
+
+        Parameters
+        ----------
+
+        column : str
+            Name of property (e.g. exon_id)
+
+        feature : str
+            Type of entry (e.g. exon)
+
+        distinct : bool, optional
+            Keep only unique values
+
+        contig : str, optional
+            Restrict query to particular contig
+
+        Returns a single column Pandas DataFrame constructed from query results.
+        """
+        csv_path = self.local_csv_path(
+            feature=feature,
+            column=column,
+            contig=contig)
+        def run_query():
+            values = self._query_feature_values(
+                column=column,
+                feature=feature,
+                distinct=distinct,
+                contig=contig)
+            return pd.DataFrame({column : values})
+        return memory_cache.load_csv(csv_path, expensive_action=run_query)
 
     def _query_distinct_on_contig(self, column_name, feature, contig):
         contig = normalize_chromosome(contig)
@@ -587,7 +527,7 @@ class EnsemblRelease(object):
     ###################################################
 
     def gene_by_id(self, gene_id):
-        return Gene(gene_id, self.db())
+        return Gene(gene_id, self.db)
 
     def genes_by_name(self, gene_name):
         gene_ids = self.gene_ids_of_gene_name(gene_name)
@@ -614,7 +554,7 @@ class EnsemblRelease(object):
         return str(results[0][0])
 
     def gene_names(self):
-        return self._query_feature('gene_name', 'gene')
+        return self._all_feature_values('gene_name', 'gene')
 
     def gene_names_on_contig(self, contig):
         """
@@ -646,7 +586,7 @@ class EnsemblRelease(object):
     ###################################################
 
     def gene_ids(self):
-        return self._query_feature('gene_id', 'gene')
+        return self._all_feature_values('gene_id', 'gene')
 
     def gene_ids_on_contig(self, contig):
         """
@@ -686,8 +626,22 @@ class EnsemblRelease(object):
     #
     ###################################################
 
+    def transcripts(self, contig=None):
+        """
+        Construct Transcript object for every transcript entry in
+        the Ensembl database. Optionally restrict to a particular
+        chromosome using the `contig` argument.
+        """
+        # DataFrame with single column 'transcript_id'
+        transcript_ids_df = self.transcript_ids(contig=contig)
+        db = self.db
+        return [
+            Transcript(transcript_id, db)
+            for transcript_id in transcript_ids_df['transcript_id']
+        ]
+
     def transcript_by_id(self, transcript_id):
-        return Transcript(transcript_id, self.db())
+        return Transcript(transcript_id, self.db)
 
     def transcripts_by_name(self, transcript_name):
         transcript_ids = self.transcript_ids_of_transcript_name(transcript_name)
@@ -696,6 +650,23 @@ class EnsemblRelease(object):
             for transcript_id in transcript_ids
         ]
 
+    def transcript_by_name(self, transcript_name):
+        """
+        Get the single transcript associated with a particular name,
+        raise an exception if there are zero or multiple transcripts.
+        """
+        transcripts = self.transcripts_by_name(transcript_name)
+
+        if len(transcripts) == 0:
+            raise ValueError(
+                "No transcripts found with name = %s" % transcript_name)
+        elif len(transcripts) > 1:
+            raise ValueError(
+                "Multiple transcripts found with name = %s (IDs = %s)" %(
+                    transcript_name,
+                    [transcript.id for transcript in transcripts]))
+        return transcripts[0]
+
     def transcript_by_protein_id(self, protein_id):
         transcript_id = self.transcript_id_of_protein_id(protein_id)
         return self.transcript_by_id(transcript_id)
@@ -716,8 +687,11 @@ class EnsemblRelease(object):
             required=True)
         return [result[0] for result in results]
 
-    def transcript_names(self):
-        return self._query_feature('transcript_name', 'transcript')
+    def transcript_names(self, contig=None):
+        return self._all_feature_values(
+            column='transcript_name',
+            feature='transcript',
+            contig=contig)
 
     def transcript_names_on_contig(self, contig):
         """
@@ -728,11 +702,18 @@ class EnsemblRelease(object):
             feature='transcript',
             contig=contig)
 
-    def transcript_name_of_gene_name(self, gene_name):
-        return _query_transcript_names('gene_name', gene_name)[0]
+    def transcript_names_of_gene_name(self, gene_name):
+        return self._query_transcript_names('gene_name', gene_name)
 
     def transcript_name_of_transcript_id(self, transcript_id):
-        return _query_transcript_names('transcript_id', transcript_id)[0]
+        transcript_names = self._query_transcript_names(
+            'transcript_id', transcript_id)
+        if len(transcript_names) == 0:
+            raise ValueError(
+                "No transcript names for transcript ID = %s" % transcript_id)
+        assert len(transcript_names) == 1, \
+            "Multiple transcript names for transcript ID = %s" % transcript_id
+        return transcript_names[0]
 
     ###################################################
     #
@@ -750,6 +731,12 @@ class EnsemblRelease(object):
             required=True)
         return [result[0] for result in results]
 
+    def transcript_ids(self, contig=None):
+        return self._all_feature_values(
+            column='transcript_id',
+            feature='transcript',
+            contig=contig)
+
     def transcript_ids_of_gene_id(self, gene_id):
         return self._query_transcript_ids('gene_id', gene_id)
 
@@ -762,6 +749,35 @@ class EnsemblRelease(object):
     def transcript_ids_of_exon_id(self, exon_id):
         return self._query_transcript_ids('exon_id', exon_id)
 
+    ###################################################
+    #
+    #             Exon Info Objects
+    #
+    ###################################################
+
+    def exons(self, contig=None):
+        """
+        Create exon object for all exons in the database, optionally
+        restrict to a particular chromosome using the `contig` argument.
+        """
+        # DataFrame with single column called 'exon_id'
+        exon_ids_df = self.exon_ids(contig=contig)
+        db = self.db
+        return [Exon(exon_id, db) for exon_id in exon_ids_df['exon_id']]
+
+    def exon_by_id(self, exon_id):
+        return Exon(exon_id, self.db)
+
+    def exon_by_transcript_id_and_number(self, transcript_id, exon_number):
+        transcript = self.transcript_by_id(transcript_id)
+        if len(transcript.exons) > exon_number:
+            raise ValueError(
+                "Invalid exon number for transcript %s" % transcript_id)
+
+        # exon numbers in Ensembl are 1-based, need to subtract 1 to get
+        # a list index
+        return transcript.exons[exon_number - 1]
+
     ###################################################
     #
     #                Exon IDs
@@ -778,8 +794,11 @@ class EnsemblRelease(object):
             required=True)
         return [result[0] for result in results]
 
-    def exon_ids(self):
-        return self._query_feature('exon_id', 'exon')
+    def exon_ids(self, contig=None):
+        return self._all_feature_values(
+            column='exon_id',
+            feature='exon',
+            contig=contig)
 
     def exon_ids_of_gene_id(self, gene_id):
         return self._query_exon_ids('gene_id', gene_id)
@@ -842,4 +861,3 @@ class EnsemblRelease(object):
 
 
 
-
