@@ -18,7 +18,7 @@ def test_gene_ids_hla_a():
     assert ids == ['ENSG00000206503'], ids
 
 def test_transcript_ids_hla_a():
-    transcript_ids = ensembl.transcript_ids_at_loci(6, 29941260, 29945884)
+    transcript_ids = ensembl.transcript_ids_at_locus(6, 29941260, 29945884)
     expected = [
         'ENST00000396634',
         'ENST00000376809',
